using System;
using System.Data.Odbc;

namespace ClassLibraryClinica
{
	/// <summary>
	/// Descripcin breve de FacturaCliente.
	/// </summary>
	public class FacturaCliente
	{
		private int idFacturaCliente;
		private int idConsulta;
		private DateTime fecha;
		private float descuento;
		private bool emitida;
		private bool pagada;
		private string formaPago;

		public FacturaCliente()
		{
			Limpiar();
		}

		public int gsIdFacturaCliente
		{
			get { return idFacturaCliente; }
			set { idFacturaCliente = value; }
		}

		public int gsIdConsulta
		{
			get { return idConsulta; }
			set { idConsulta = value; }
		}

		public DateTime gsFecha
		{
			get { return fecha; }
			set { fecha = value; }
		}

		public float gsDescuento
		{
			get { return descuento; }
			set { descuento = value; }
		}

		public bool gsEmitida
		{
			get { return emitida; }
			set { emitida = value; }
		}

		public bool gsPagada
		{
			get { return pagada; }
			set { pagada = value; }
		}

		public string gsFormaPago
		{
			get { return formaPago; }
			set { formaPago = value; }
		}

		public void Limpiar()
		{
			gsIdFacturaCliente = 0;
			gsIdConsulta = 0;
			gsFecha = new DateTime(1900, 1, 1);
			gsDescuento = -1;
			gsEmitida = false;
			gsPagada = false;
			gsFormaPago = "";
		}

		public int AgregarDB()
		{
			Consulta consulta = new Consulta();

			Utilidades utls = new Utilidades();
			string fecha = utls.FechaPostgres(gsFecha);

			string sql = "";
			sql += "insert into facturascliente (idconsulta, fecha, descuento, emitida, pagada, formapago) values (";
			sql += (gsIdConsulta == 0)? "NULL, " : gsIdConsulta.ToString() + ", ";
			sql += (fecha.Equals(""))? "NULL, " : "'" + fecha + "', ";
			sql += (gsDescuento < 0)? "NULL, " : gsDescuento.ToString() + ", ";
			sql += (gsEmitida)? "'t', " : "'f', ";
			sql += (gsPagada)? "'t', " : "'f', ";
			sql += (gsFormaPago.Equals(""))? "NULL" : "'" + gsFormaPago + "'";
			sql += "); ";

			consulta.ExecNQ("begin; " + sql);
			int facturaCliente = consulta.ExecS("select currval('facturascliente_idfacturacliente_seq'); ");
			consulta.ExecNQ((facturaCliente > 0)? "commit;" : "rollback;");
			consulta.Cierre();

			return facturaCliente;
		}

		public bool ModificarDB()
		{
			Consulta consulta = new Consulta();

			Utilidades utls = new Utilidades();
			string fecha = utls.FechaPostgres(gsFecha);

			string sql = "";
			sql += "update facturascliente set ";
			sql += "idconsulta = " + (gsIdConsulta == 0? "NULL" : gsIdConsulta.ToString()) + ", ";
			sql += "fecha = " + ((fecha.Equals(""))? "NULL" : "'" + fecha + "'") + ", ";
			sql += "descuento = " + ((gsDescuento < 0)? "NULL" : gsDescuento.ToString()) + ", ";
			sql += "emitida = " + ((gsEmitida)? "'t'" : "'f'") + ", ";
			sql += "pagada = " + ((gsPagada)? "'t'" : "'f'") + ", ";
			sql += "formapago = " + ((gsFormaPago.Equals(""))? "NULL" : "'" + gsFormaPago + "'") + " ";
			sql += "where idfacturacliente = " + gsIdFacturaCliente.ToString() + "; ";

			bool error = consulta.ExecNQ(sql) <= 0;
			consulta.ExecNQ(!error? "commit;" : "rollback;");
			consulta.Cierre();

			return !error;
		}

		public bool EliminarDB(int id)
		{
			Consulta consulta = new Consulta();

			bool error = consulta.ExecNQ("delete from facturascliente where idfacturacliente = " + id.ToString() + "; ") <= 0;
			consulta.ExecNQ(!error? "commit;" : "rollback;");
			consulta.Cierre();

			return !error;		
		}

		public bool CargarIdFacturaClienteDB(int id)
		{
			return CargarDB("where idfacturacliente = " + gsIdFacturaCliente.ToString());
		}

		public bool CargarIdClienteDB(int id)
		{
			return CargarDB("where idconsulta = " + gsIdConsulta.ToString());
		}

		public bool CargarDB(string condicion)
		{
			Consulta consulta = new Consulta();

			string sql = "";
			sql += "select idfacturacliente, idconsulta, ";
			sql += "extract(year from fecha) end as anyo, ";
			sql += "extract(month from fecha) end as mes, ";
			sql += "extract(day from fecha) end as dia, ";
			sql += "descuento, emitida, pagada, formapago ";
			sql += "from facturascliente " + condicion + "; ";
			OdbcDataReader tupla = consulta.ExecR(sql);

			bool cargado = tupla.Read();
			if (cargado)
			{
				gsIdFacturaCliente = tupla.GetInt32(0);
				gsIdConsulta = tupla.GetInt32(1);
				DateTime fechaPostgres = new DateTime(
					int.Parse(tupla["anyo"].ToString()), 
					int.Parse(tupla["mes"].ToString()), 
					int.Parse(tupla["dia"].ToString())
					);
				gsFecha = fechaPostgres;
				gsDescuento = float.Parse(tupla["descuento"].ToString());
				gsEmitida = tupla["emitida"].ToString() == "t";
				gsPagada = tupla["pagada"].ToString() == "t";
				gsFormaPago = tupla["formapago"].ToString();
			}
			else Limpiar();

			consulta.Cierre();
			tupla.Close();

			return cargado;
		}
	}
}
