using System;
using System.Data.Odbc;

namespace ClassLibraryClinica
{
	/// <summary>
	/// Descripcin breve de laboratorio.
	/// </summary>
	public class Citas
	{
		private int idCita;
		private int idCliente;
		private int idEmpleado;
		private DateTime fechaHora;
		private bool confirmada;
		private bool realizada;
		private int duracionEstimada;
		private string descripcion;
	
		public Citas()
		{
			Limpiar();
		}

		public int gsIdCita
		{
			get { return idCita; }
			set { idCita = value; }
		}

		public int gsIdCliente
		{
			get { return idCliente; }
			set { idCliente = value; }
		}

		public int gsIdEmpleado
		{
			get { return idEmpleado; }
			set { idEmpleado = value; }
		}

		public DateTime gsFechaHora
		{
			get { return fechaHora; }
			set { fechaHora = value; }
		}

		public bool gsConfirmada
		{
			get { return confirmada; }
			set { confirmada = value; }
		}

		public bool gsRealizada
		{
			get { return realizada; }
			set { realizada = value; }
		}

		public int gsDuracionEstimada
		{
			get { return duracionEstimada; }
			set { duracionEstimada = value; }
		}

		public string gsDescripcion
		{
			get { return descripcion; }
			set { descripcion = value; }
		}
		
		public void Limpiar()
		{
			gsIdCita = 0;
			gsIdCliente = 0;
			gsIdEmpleado = 0;
			gsFechaHora = new DateTime(1900, 1, 1, 0, 0, 0);
			gsConfirmada = false;
			gsRealizada = false;
			gsDuracionEstimada = -1;
			gsDescripcion = "";
		}

		public int AgregarDB()
		{
			Consulta consulta = new Consulta();
			
			Utilidades utls = new Utilidades();
			string fecha = utls.FechaPostgres(gsFechaHora);
			string hora = utls.HoraPostgres(gsFechaHora);

			string sql;
			sql  = "insert into citas (idcliente, idempleado, fecha, hora, confirmada, realizada, duracionestimada, descripcion) values (";
			sql += (gsIdCliente == 0)? "NULL, " : gsIdCliente.ToString() + ", ";
			sql += (gsIdEmpleado == 0)? "NULL, " : gsIdEmpleado.ToString() + ", ";
			sql += (fecha.Equals(""))? "NULL, " : "'" + fecha + "', ";
			sql += (hora.Equals(""))? "NULL, " : "'" + hora + "', ";
			sql += (gsConfirmada)? "'t', " : "'f', ";
			sql += (gsRealizada)? "'t', " : "'f', ";
			sql += (gsDuracionEstimada < 0)? "NULL, " : gsDuracionEstimada.ToString() + ", ";
			sql += (gsDescripcion.Equals(""))? "NULL, " : "'" + gsDescripcion + "' ";
			sql += "); ";
			
			consulta.ExecNQ("begin; " + sql);
			int cita = consulta.ExecS("select currval('citas_idcita_seq'); ");
			consulta.ExecNQ((cita > 0)? "commit;" : "rollback;");
			consulta.Cierre();

			return cita;
		}

		public bool ModificarDB()
		{
			Consulta cns = new Consulta();

			Utilidades utls = new Utilidades();
			string fecha = utls.FechaPostgres(gsFechaHora);
			string hora = utls.HoraPostgres(gsFechaHora);

			string sql = "";
			sql += "update citas set ";
			sql += "idcliente = " + ((gsIdCliente == 0)? "NULL" : gsIdCliente.ToString()) + ", ";
			sql += "idempleado = " + ((gsIdEmpleado == 0)? "NULL" : gsIdEmpleado.ToString()) + ", ";
			//sql += "fecha = " + ((fecha.Equals(""))? "NULL" : "'" + fecha + "'" ) + ", ";
			//sql += "hora = " + ((hora.Equals(""))? "NULL" : "'" + hora + "'") + ", ";
			sql += "confirmada = " + ((gsConfirmada)? "'t'" : "'f'") + ", " ;
			sql += "realizada = " + ((gsRealizada)? "'t'" : "'f'") + ", ";
			sql += "duracionestimada = " + ((gsDuracionEstimada < 0)? "NULL" : gsDuracionEstimada.ToString()) + ", ";
			sql += "descripcion = " + ((gsDescripcion.Equals(""))? "NULL " : "'" + gsDescripcion + "' " );
			sql += "where idcita = " + gsIdCita.ToString() + "; ";

			bool error = cns.ExecNQ(sql) <= 0;
			cns.ExecNQ(!error? "commit;" : "rollback;");
			cns.Cierre();

			return !error;
		}

		public bool EliminarDB(int id)
		{
			Consulta cns = new Consulta();

			bool error = cns.ExecNQ("delete from citas where idcita = " + id.ToString() + "; ") <= 0;
			cns.ExecNQ(!error? "commit;" : "rollback;");
			cns.Cierre();

			return !error;
		}

		public bool cargarDB(int id)
		{
			Consulta consulta = new Consulta();

			string sql = "";
			sql += "select idcita, idcliente, idempleado, ";
			sql += "extract(year from fecha) as anyo, ";
			sql += "extract(month from fecha) as mes, ";
			sql += "extract(day from fecha) as dia, ";
			sql += "extract(hour from hora) as hora, ";
			sql += "extract(minute from hora) as minutos, ";
			sql += "extract(second from hora) as segundos, ";
			sql += "confirmada, realizada, duracionestimada, descripcion ";
			sql += "from citas where idcita = " + id.ToString() + "; ";
			
			OdbcDataReader tupla = consulta.ExecR(sql);
			
			bool cargado = tupla.Read();
			if (cargado)
			{
				gsIdCita = tupla.GetInt32(0);
				gsIdCliente = tupla.GetInt32(1);
				gsIdEmpleado = tupla.GetInt32(2);
				/*DateTime fechaHoraPostgres = new DateTime(
					int.Parse(tupla["anyo"].ToString()), 
					int.Parse(tupla["mes"].ToString()), 
					int.Parse(tupla["dia"].ToString()),
					int.Parse(tupla["hora"].ToString()),
					int.Parse(tupla["minutos"].ToString()),
					int.Parse(tupla["segundos"].ToString())
					);*/
				//gsFechaHora = fechaHoraPostgres;
				gsConfirmada = tupla["confirmada"].ToString() == "t";
				gsRealizada = tupla["realizada"].ToString() == "t";
				gsDuracionEstimada = int.Parse(tupla["duracionestimada"].ToString());
				gsDescripcion = tupla["descripcion"].ToString();
			}
			else Limpiar();

			consulta.Cierre();
			tupla.Close();

			return cargado;
		}
	}
}

