using System;
using System.Data.Odbc;

namespace ClassLibraryClinica
{
	/// <summary>
	/// Descripcin breve de FacturaLaboratorio.
	/// </summary>
	public class FacturaLaboratorio
	{
		private int idFacturaLaboratorio;
		private int idPedido;
		private DateTime fecha;
		private bool recibida;
		private bool pagada;

		public FacturaLaboratorio()
		{
			Limpiar();
		}

		public int gsIdFacturaLaboratorio
		{
			get { return idFacturaLaboratorio; }
			set { idFacturaLaboratorio = value; }
		}

		public int gsIdPedido
		{
			get { return idPedido; }
			set { idPedido = value; }
		}

		public DateTime gsFecha
		{
			get { return fecha; }
			set { fecha = value; }
		}

		public bool gsRecibida
		{
			get { return recibida; }
			set { recibida = value; }
		}

		public bool gsPagada
		{
			get { return pagada; }
			set { pagada = value; }
		}

		public void Limpiar()
		{
			gsIdFacturaLaboratorio = 0;
			gsIdPedido = 0;
			gsFecha = new DateTime(1900, 1, 1);
			gsRecibida = false;
			gsPagada = false;
		}

		public int Agregar()
		{
			Consulta consulta = new Consulta();

			Utilidades utls = new Utilidades();
			string fecha = utls.FechaPostgres(gsFecha);

			string sql = "";
			sql += "insert into facturaslaboratorio (idpedido, fecha, recibida, pagada) values (";
			sql += (gsIdPedido == 0)? "NULL, " : gsIdPedido.ToString() + ", ";
			sql += (fecha.Equals(""))? "NULL, " : "'" + fecha + "', ";
			sql += (gsRecibida)? "'t', " : "'f', ";
			sql += (gsPagada)? "'t'" : "'f'";
			sql += "); ";

			consulta.ExecNQ("begin; " + sql);
			int facturaLaboratorio = consulta.ExecS("select currval('facturaslaboratorio_idfacturalaboratorio_seq'); ");
			consulta.ExecNQ((facturaLaboratorio > 0)? "commit;" : "rollback;");
			consulta.Cierre();

			return facturaLaboratorio;
		}

		public bool Modificar()
		{
			Consulta consulta = new Consulta();

			Utilidades utls = new Utilidades();
			string fecha = utls.FechaPostgres(gsFecha);

			string sql = "";
			sql += "update facturaslaboratorio set ";
			sql += "idpedido = " + (gsIdPedido == 0? "NULL" : gsIdPedido.ToString()) + ", ";
			sql += "fecha = " + ((fecha.Equals(""))? "NULL" : "'" + fecha + "'") + ", ";
			sql += "recibida = " + ((gsRecibida)? "'t'" : "'f'") + ", ";
			sql += "pagada = " + ((gsPagada)? "'t'" : "'f'") + " ";
			sql += "where idfacturalaboratorio = " + gsIdFacturaLaboratorio.ToString() + "; ";

			bool error = consulta.ExecNQ(sql) <= 0;
			consulta.ExecNQ(!error? "commit;" : "rollback;");
			consulta.Cierre();

			return !error;
		}

		public bool Eliminar(int id)
		{
			Consulta consulta = new Consulta();

			bool error = consulta.ExecNQ("delete from facturaslaboratorio where idfacturalaboratorio = " + id.ToString() + "; ") <= 0;
			consulta.ExecNQ(!error? "commit;" : "rollback;");
			consulta.Cierre();

			return !error;	
		}

		public bool CargarIdFacturaLaboratorioDB(int id)
		{
			return CargarDB("where idfacturalaboratorio = " + id.ToString());
		}

		public bool CargarIdPedido(int id)
		{
			return CargarDB("where idpedido = " + id.ToString());
		}

		public bool CargarDB(string condicion)
		{
			Consulta consulta = new Consulta();

			string sql = "";
			sql += "select idfacturalaboratorio, idpedido, ";
			sql += "extract(year from fecha) end as anyo, ";
			sql += "extract(month from fecha) end as mes, ";
			sql += "extract(day from fecha) end as dia, ";
			sql += "recibida, pagada ";
			sql += "from facturaslaboratorio " + condicion + "; ";
			OdbcDataReader tupla = consulta.ExecR(sql);

			bool cargado = tupla.Read();
			if (cargado)
			{
				gsIdFacturaLaboratorio = tupla.GetInt32(0);
				gsIdPedido = tupla.GetInt32(1);
				DateTime fechaPostgres = new DateTime(
					int.Parse(tupla["anyo"].ToString()), 
					int.Parse(tupla["mes"].ToString()), 
					int.Parse(tupla["dia"].ToString())
					);
				gsFecha = fechaPostgres;
				gsRecibida = tupla["recibida"].ToString() == "t";
				gsPagada = tupla["pagada"].ToString() == "t";
			}
			else Limpiar();

			consulta.Cierre();
			tupla.Close();

			return cargado;
		}
	}
}
